/* ***** BEGIN LICENSE BLOCK *****

 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * The Original Code is bookmarkshome
 *
 * The Initial Developer of the Original Code is Jeroen Groenenboom.
 *
 * Contributor(s):
 *   Jean-Bernard Marcon <jean-bernard.marcon@laposte.net>
 *   Alex Sirota 
 *
 * ***** END LICENSE BLOCK ***** */

/*
   id's of elements in document are equal to corresponding preferences
 */



var bmhOptions = null;
var bmhRDF = null;

function BmhOptions() {
    // general
    this.hideEnginesField = document.getElementById( "hideSearchEngines" );
    this.engineMenu = document.getElementById( "engineMenu" );
    this.dfltButton = document.getElementById( "defaultEngineButton" );
    this.nrOfColumnsField = document.getElementById( "nrOfColumns" );
    this.maxNameLengthField = document.getElementById( "maxNameLength" );
    this.maxHeaderLinesField = document.getElementById( "maxHeaderLines" );
    this.iconsMenu = document.getElementById( "iconDisplay" );
    this.openFolderInGroup = document.getElementById( "openFolderIn" );
    this.browser = bmhLib.getMainWindow().getBrowser().selectedBrowser;
    // style
    this.stylesheetField = document.getElementById( "stylesheet" );
    this.stylesArray = new Array();
    this.stylesMenu = document.getElementById( "styles" );
    this.stylesDS = null;
    // folders 
    this.excludeMenu = document.getElementById( "excludeIndex" );
    this.contentFrame = document.getElementById( "folderFrame" ).contentWindow;
    this.inOrExcludedFoldersBox = this.contentFrame.document.getElementById( "inOrExcludedFolders" );
    this.subfolderCheckbox = document.getElementById( "subfolderCheckbox" );
    this.folderCheckboxArray = new Array();
    // minimum values for integer text fields
    this.minValues = new Array();
    this.minValues["nrOfColumns"] = 1;
    this.minValues["maxNameLength"] = 1;
    this.minValues["maxHeaderLines"] = 1;
    this.init();
}


BmhOptions.prototype = {

init: function() {
	  // create RDF object
	  bmhRDF = new BmhRDF();
	  // general
	  this.hideEnginesField.checked = bmhLib.getPref( "hideSearchEngines" );
	  this.hiddenEngines = bmhLib.getArrayFromPref( "hiddenEngines" );
	  // check if default engine hasn't been removed
	  this.checkDefaultEngine( bmhLib.getPref( "engineURL" ));
	  this.nrOfColumnsField.value = bmhLib.getPref( "nrOfColumns" );
	  this.maxNameLengthField.value = bmhLib.getPref( "maxNameLength" );
	  this.maxHeaderLinesField.value = bmhLib.getPref( "maxHeaderLines" );
	  this.iconsMenu.selectedIndex = bmhLib.getPref( "iconDisplay" );
	  this.openFolderInGroup.selectedIndex = bmhLib.getPref( "openFolderIn" );
	  // style
	  this.stylesheetField.action = 1;
	  this.initStyle();
	  // folders
	  this.excludeMenu.selectedIndex = bmhLib.getPref( "excludeIndex" );
	  this.parseBookmarks();

      },

removeEngine: function() {
		  const selectedItem = this.engineMenu.selectedItem;
		  if( ! selectedItem ) {
		      return;
		  }
		  const engineValue = selectedItem.getAttribute( "value" ); 
		  const engineName = selectedItem.getAttribute( "label" );
		  var enginePath = engineValue;
		  var imgPath = selectedItem.getAttribute( "src" ); 
		  // remove 'engine://' and unescape path
		  enginePath = decodeURIComponent( enginePath.substring( 9 ));
		  try {
		      var handler = Components.classes["@mozilla.org/network/protocol;1?name=file"]
			  .createInstance( Components.interfaces.nsIFileProtocolHandler );
		      // convert to native path spec: 
		      imgPath = handler.getFileFromURLSpec( imgPath ).path;
		  }
		  catch( e ) {}
		  if( ! this.getPromptService().confirm( window, bmhLib.getLocaleString( "DeletePluginString" ), 
			      bmhLib.getLocaleString( "DeletePluginPrompt", [engineName, enginePath, imgPath] ))) {
		      return;
		  }
		  // delete .src and image file
		  if( enginePath != "" ) {
		      this.deleteFile( enginePath );
		  }
		  if( imgPath != "" ) {
		      this.deleteFile( imgPath );
		  }
		  if( bmhLib.testProps( this.browser, ["contentDocument","title"] ) == "BookmarksHome" ) {
		      // we're still seeing bmh 
		      this.delEngineFromBmhPage( engineValue );
		  }
		  // remove engine from options window
		  this.delEngineFromMenu( selectedItem );
		  // remove engine from searchbar
		  this.delEngineFromSearchBar( engineValue );
		  // remove engine from internet search data source
		  this.delEngineFromDS( engineValue );
	      },

delEngineFromDS: function( engineValue ) {
		     try {
			 const delEngineResource = bmhRDF.RDF.GetResource( engineValue );
			 bmhRDF.ISDS.Unassert( bmhRDF.rootEngine, bmhRDF.childArc, delEngineResource ); 
		     }
		     catch( e ) {
			 // do nothing, engine will be removed from DS after restart
		     }
		 },


delEngineFromMenu: function( engineItem ) {
			  engineItem.parentNode.removeChild( engineItem );
			  if( this.dfltEngine ) {
			      this.checkDefaultEngine( this.dfltEngine.value );
			  }
			  else {
			      // create new default
			      this.checkDefaultEngine( "" );
			  }
		      },

delEngineFromSearchBar: function( engineValue ) {
			    try {
				var mainDoc = bmhLib.getMainWindow().document;
				var searchBar = mainDoc.getElementById( "searchbar" );
				var barPopup = mainDoc.getAnonymousElementByAttribute ( searchBar, "anonid", "searchbar-popup" );
				var selBarItem = searchBar.getAttribute( "searchengine" );
				var delBarItems = barPopup.getElementsByAttribute( "value", engineValue );
				if( delBarItems.length == 1 ) {
				    // to be removed engine found
				    if( selBarItem == engineValue ) {
					// to be removed engine is selected
					var barItems = barPopup.getElementsByTagName( "menuitem" );
					var nextBarItem = delBarItems[0].nextSibling;
					if( barItems.length >= 3 ) {
					    // at least one more engine. first has value "rdf:*"
					    // (addengine item appended on popup)
					    if( nextBarItem ) {
						searchBar.setAttribute( "src", nextBarItem.getAttribute( "src" ));
						searchBar.setAttribute( "searchengine", nextBarItem.getAttribute( "value" ));
					    }
					    else {
						// to be removed item is last item
						searchBar.setAttribute( "src", barItems[1].getAttribute( "src" ));
						searchBar.setAttribute( "searchengine", barItems[1].getAttribute( "value" ));
					    }
					}
					else {
					    // no more engines
					    searchBar.setAttribute( "src", null );
					    searchBar.setAttribute( "searchengine", null );
					}
				    }
				    // remove engine from menu
				    barPopup.removeChild( delBarItems[0] );
				}
			    }
			    catch( e ) {}
			},

delEngineFromBmhPage: function( engineValue ) {
			  var option = this.findEngineOption( engineValue );
			  if( option ) {
			      var engineSelect = option.parentNode;
			      engineSelect.removeChild( option );
			      // delEngineFromMenu calls bmhLib.selectEngine
			  }
		      },

findEngineOption: function( engineValue ) {
		      if( ! engineValue ) {
			  return null;
		      }
		      var htmlDoc = bmhLib.testProps( this.browser, ["contentDocument"] );
		      if( bmhLib.testProps( htmlDoc, ["title"] ) == "BookmarksHome" ) {
			  var engineSelect = htmlDoc.getElementById( "engineSelect" );
			  var options = engineSelect.getElementsByTagName( "option" );
			  for( var i = 0; i < options.length; i++ ) {
			      if( options[i].value.substr( 0, engineValue.length ) == engineValue ) {
				  return options[i]
			      }
			  }
		      }
		      return null;
		  },

checkDefaultEngine: function( value ) {
			if( value == "" ) {
			    // values at startup may be ""
			    value = "bmh";
			}
			// check if default engine hasn't been removed
			var temp = this.engineMenu.getElementsByAttribute( "value", value );
			if(( this.engineMenu.getElementsByTagName( "menuitem" ).length >= 2 )) {
			    // at least one engine present. first item has value "rdf:*"
			    if( temp.length == 0 ) {
				// default engine removed. set top engine as default
				this.engineMenu.selectedIndex = 1;

			    }
			    else {
				this.engineMenu.selectedItem = temp[0];
			    }
			    this.setDefaultEngine();
			}	  
			else {
			    // zero engines
			    this.dfltEngine = null;
			    this.dfltButton.setAttribute( "label", "" );
			    this.dfltButton.setAttribute( "image", "" );
			    this.engineMenu.selectedItem = null;
			    bmhLib.selectEngine( null );
			}
		    },

deleteFile: function ( fileName ) {
		try {
		    var file  = Components.classes["@mozilla.org/file/local;1"]
			.createInstance( Components.interfaces.nsILocalFile );
		    file.initWithPath( fileName );
		    file.remove( false );
		}
		catch( e ) {
		    alert( bmhLib.getLocaleString( "FileProblemString",[e.toString(),fileName] ));
		}
	    },

apply: function() {
	   try {
	       var prefs = new Array( bmhLib.getNrOfPrefs() );
	       var prefIndex = -1;
	       // general
	       prefs[++prefIndex] = this.parseValue( this.hideEnginesField.id, this.hideEnginesField.checked );
	       var hiddenEnginesString = bmhLib.getPrefFromArray( this.hiddenEngines );
	       prefs[++prefIndex] = this.parseValue( "hiddenEngines", hiddenEnginesString );
	       prefs[++prefIndex] = this.parseValue( "engineURL", ( this.dfltEngine ? this.dfltEngine.value : "" ));
	       prefs[++prefIndex] = this.parseValue( this.nrOfColumnsField.id, this.nrOfColumnsField.value );
	       prefs[++prefIndex] = this.parseValue( this.maxNameLengthField.id, this.maxNameLengthField.value );
	       prefs[++prefIndex] = this.parseValue( this.openFolderInGroup.id, this.openFolderInGroup.selectedIndex );
	       prefs[++prefIndex] = this.parseValue( this.iconsMenu.id, this.iconsMenu.selectedIndex );
	       prefs[++prefIndex] = this.parseValue( this.maxHeaderLinesField.id, this.maxHeaderLinesField.value );
	       // style
	       // apply unsaved changes to current sheet
	       this.changeSheetProperty( this.getCurStyleName(), this.stylesMenu.selectedIndex,
		       this.stylesheetField.value );
	       prefs[++prefIndex] = this.parseValue( "styleName", this.getCurStyleName() );

	       // folders
	       prefs[++prefIndex] = this.parseValue( this.excludeMenu.id, this.excludeMenu.selectedIndex );
	       // get checked folder checkboxes
	       var checkedBoxes = this.contentFrame.document.getElementsByAttribute( "checked", "true" );
	       var sortedInOrExcludedFolders = new Array( checkedBoxes.length ); 
	       var checkedIndex;
	       // get ancestor string for checked folder checkboxes
	       for( var i = 0; i < checkedBoxes.length; i++ ) {
		   checkedIndex = parseInt( checkedBoxes[i].id.substr( 3 ));
		   sortedInOrExcludedFolders[i] = this.folderCheckboxArray[checkedIndex].value.toString();
	       }
	       // sort excluded folders for binary search
	       sortedInOrExcludedFolders.sort();
	       var inOrExcludedString = bmhLib.getPrefFromArray( sortedInOrExcludedFolders );
	       prefs[++prefIndex] = this.parseValue( this.inOrExcludedFoldersBox.id, inOrExcludedString );
	       // set prefs
	       for( prefIndex = 0; prefIndex < prefs.length; prefIndex++ ) {
		   bmhLib.setPref( prefs[prefIndex].name, prefs[prefIndex].type, prefs[prefIndex].value );
	       }
	       // write stylesfile
	       this.flushStylesDS();
	       // apply prefs. Only reload tab from which options were opened
	       // and only when bmh is still there
	       if( bmhLib.testProps( this.browser, ["contentDocument","title"] ) == "BookmarksHome" ) {
		   this.browser.reload();
	       }
	   }
	   catch( e ) {
	       // improper value in field
	       alert( e );
	   }
       },

parseValue: function( aName, aValue ) {
		var aType = bmhLib.getPrefType( aName );
		var minVal = this.minValues[aName];
		//var impropervalue = document.getElementById( "bmhStrings" ).getString( "ImproperValueString" );
		var impropervalue = bmhLib.getLocaleString( "ImproperValueString" );
		if( minVal == null ) {
		    // value does not need to be checked
		    return { name: aName, type: aType, value: aValue };
		}
		aValue = parseInt( aValue );
		if( isNaN( aValue ) || aValue < minVal ) {
		    throw aName + " " + impropervalue;
		}
		return { name: aName, type: aType, value: aValue };
	    },

close: function() {
	   // get styles datasource
	   this.stylesDS = bmhLib.getStylesDS( bmhRDF );
	   var stylesList = bmhRDF.RDF.GetResource( bmhRDF.stylesNS + "stylesList" );
	   bmhRDF.RDFC.Init( this.stylesDS, stylesList );
	   var stylesEnum = bmhRDF.RDFC.GetElements();
	   var promptService = this.getPromptService(); 
	   var flags = promptService.BUTTON_TITLE_DONT_SAVE * promptService.BUTTON_POS_0;
	   flags += promptService.BUTTON_TITLE_SAVE * promptService.BUTTON_POS_1;
	   var curNode, curName, oldSheet, newSheet; 
	   var index = 0, saveasstring, savechangesstring;
	   // check all styles for changes
	   while( stylesEnum.hasMoreElements() ) {
	       curNode = stylesEnum.getNext();
	       oldSheet = bmhLib.getProperty( curNode, bmhRDF.sheetArc, this.stylesDS );
	       newSheet = this.stylesArray[index].sheet; 
	       if( newSheet.replace( /\s*/g, "" )  !=  oldSheet.replace( /\s*/g, "" )) {
		   // sheet has changed
		   curName = this.stylesArray[index].name;
		   //saveasstring = document.getElementById( "bmhStrings" ).getString( "SaveChangesToString" );
		   saveasstring = bmhLib.getLocaleString( "SaveChangesToString" );
		   //savechangesstring = document.getElementById( "bmhStrings" ).getString( "SaveChangesString" );
		   savechangesstring = bmhLib.getLocaleString( "SaveChangesString" );
		   message = saveasstring + " " + curName + "?";
		   if( promptService.confirmEx(window, savechangesstring, 
			       message, flags, null, null, null, null, { value: 0 } )) {
		       this.changeSheetProperty( curName, index, newSheet );
		   }
	       }
	       index += 1;
	   }
       },

hideEngines: function() {
		 try {
		     var doc = bmhLib.testProps( this.browser, ["contentDocument"] );
		     if( bmhLib.testProps( doc, ["title"] ) == "BookmarksHome" ) {
			 if( ! this.hideEnginesField.checked ) {
			     // check if not all engines are hidden individually
			     var dfltOption = this.findEngineOption( this.dfltEngine.value );
			     bmhLib.selectEngine( dfltOption );
			 }
			 else {
			     doc.getElementById( "searchForm" ).style.display = "none";
			 }
		     }
		 }
		 catch( e ) {}
	     },

toggleEngineDisplay: function() {
			 var selectedItem = bmhLib.testProps( this.engineMenu, ["selectedItem"] );
			 if( ! selectedItem ) {
			     return;
			 }
			 var option = this.findEngineOption( selectedItem.value );
			 var index = bmhLib.binarySearch( selectedItem.value, this.hiddenEngines );
			 if( index >= 0 ) {
			     // engine in hidden engines list, remove it and show option
			     this.hiddenEngines.splice( index, 1 );
			     if( option ) {
				 option.style.display = "";
			     }
			 }
			 else {
			     // engine not in list. Add and hide option
			     this.hiddenEngines.push( selectedItem.value );
			     this.hiddenEngines.sort();
			     if( option ) {
				 option.style.display = "none";
			     }
			 }
			 var dfltOption = this.findEngineOption( this.dfltEngine.value );
			 if( bmhLib.testProps( this.browser, ["contentDocument","title"] ) == "BookmarksHome" ) {
			     bmhLib.selectEngine( dfltOption );
			 }
		     },

setDefaultEngine: function() {
		      var selectedItem = bmhLib.testProps( this.engineMenu, ["selectedItem"] );
		      if( ! selectedItem ) {
			  return;
		      }
		      this.dfltButton.setAttribute( "image", selectedItem.getAttribute( "src" ));
		      this.dfltButton.setAttribute( "label", selectedItem.getAttribute( "label" ));
		      this.dfltEngine = selectedItem;
		      if( this.hideEnginesField.checked ) {
			  return;
		      }
		      else if( bmhLib.testProps( this.browser, ["contentDocument","title"] ) == "BookmarksHome"  ) {
			  var dfltOption = this.findEngineOption( selectedItem.value );
			  bmhLib.selectEngine( dfltOption );
		      }
		  },


setIconDisplay: function( displayIndex ) {
		    var i, icons;
		    if( bmhLib.testProps( this.browser, ["contentDocument","title"] ) == "BookmarksHome" ) {
			icons = this.browser.contentDocument.getElementsByTagName( "img" );
			for( i = 0; i < icons.length; i++ ) {
			    if( icons[i].getAttribute( "class" ) == "normalIcon" ) {
				icons[i].style.display = bmhLib.getIconDisplay( "Folder", displayIndex );
			    }
			    else if( icons[i].getAttribute( "class" ) == "LivemarkIcon" ) {
				icons[i].style.display = bmhLib.getIconDisplay( "Livemark", displayIndex );
			    }
			}
		    }
		},

initStyle: function() {
	       // get styles datasource
	       this.stylesDS = bmhLib.getStylesDS( bmhRDF );
	       // RDF variables
	       var stylesList = bmhRDF.RDF.GetResource( bmhRDF.stylesNS + "stylesList" );
	       bmhRDF.RDFC.Init( this.stylesDS, stylesList );
	       var stylesEnum = bmhRDF.RDFC.GetElements();
	       // other variables
	       var stylesMenupopup = document.getElementById( "sheetNames" ); 
	       var prefStyleName = bmhLib.getPref( "styleName" );
	       var curNode, curNameNode, curSheetNode, stylesMenuitem; 
	       var prefIndex = 0;
	       var index = 0;
	       // fill menu and array with styles
	       while( stylesEnum.hasMoreElements() ) {
		   curNode = stylesEnum.getNext();
		   curNameNode = this.stylesDS.GetTarget( curNode, bmhRDF.nameArc, true );
		   if( curNameNode instanceof Components.interfaces.nsIRDFLiteral ) {
		       stylesMenuitem = document.createElement( "menuitem" );
		       stylesMenuitem.setAttribute( "label", curNameNode.Value );
		       stylesMenupopup.appendChild( stylesMenuitem );
		       if( curNameNode.Value == prefStyleName ) {
			   // determine index of preferred style
			   prefIndex = index;
		       }
		       curSheetNode = this.stylesDS.GetTarget( curNode, bmhRDF.sheetArc, true );
		       if( curSheetNode instanceof Components.interfaces.nsIRDFLiteral ) {
			   this.stylesArray[index] = { name: curNameNode.Value, sheet: 
			       bmhLib.formatSheet( curSheetNode.Value ) };
			   index += 1;
		       }
		   }
	       }
	       // set elements to preferred style
	       if( this.stylesArray.length >= 0 ) {
		   this.stylesMenu.selectedIndex = prefIndex; 
		   this.stylesheetField.value = this.stylesArray[prefIndex].sheet;
	       }
	       // set color in colorpicker
	       var color = "#FF0000";
	       document.getElementsByTagName( "colorpicker" )[0].color = color;
	       this.showColor( color ); 
	   },

flushStylesDS: function() {
		   try {
		       var stylesFile = bmhLib.getStylesFile();
		       var stylesFileURL = bmhLib.getStylesFileURL( stylesFile );
		       this.stylesDS.QueryInterface( Components.interfaces.nsIRDFRemoteDataSource )
			   .FlushTo( stylesFileURL );
		   }
		   catch( e ) {}
	       },



updateStyle: function( action ) {
		 switch( action ) {
		     case 1:
			 // update style with timeout, set action prop to 2 
			 this.stylesheetField.action = 2;
			 window.setTimeout( "bmhOptions.updateStyle( 3 )", 200 );
			 break;
		     case 2:
			 // waiting for update, do nothing  
			 break;
		     case 3: 
			 // execute update, set action prop to 1
			 if( bmhLib.testProps( this.browser, ["contentDocument","title"] ) == "BookmarksHome" ) {
			     var styleNode = this.browser.contentDocument.getElementsByTagName( "style" )[0];
			     styleNode.innerHTML = this.stylesheetField.value;
			     // update array so that changes will be preserved as long as
			     // options window is open
			     this.stylesArray[this.stylesMenu.selectedIndex].sheet = this.stylesheetField.value;
			     this.stylesheetField.action = 1;
			 }
			 break;
		 }
	     },

changeSheetProperty: function( name, index, sheetStr ) { 
			 var sheetArc = bmhRDF.RDF.GetResource( bmhRDF.stylesNS + "sheet" );
			 // get resources for style
			 var styleResource = bmhRDF.RDF.GetResource( bmhRDF.stylesNS + name );
			 var oldSheet = this.stylesDS.GetTarget( styleResource, sheetArc, true );
			 var newSheet = bmhRDF.RDF.GetLiteral( sheetStr );
			 // change styles datasource
			 this.stylesDS.Change( styleResource, sheetArc, oldSheet, newSheet );
			 // change styles array
			 this.stylesArray[index].sheet = sheetStr;
			 // change styles menu
			 this.stylesMenu.selectedIndex = index;
		     },

getPromptService: function() {
		      return Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
			  .getService( Components.interfaces.nsIPromptService );
		  },

savePrompt: function( nameStr, sheet ) {
		var promptService = this.getPromptService();
		var name = { value: nameStr };
		var saveStyleAsStr = bmhLib.getLocaleString( "SaveStyleAsString" );
		var styleName = bmhLib.getLocaleString( "StyleNameString" );
		var replaceOk, i, styleExists, styleAlready;
		// prompt until user has given new name or wants to change the style 
		// of existing name
		while( promptService.prompt( window, saveStyleAsStr, styleName, 
			    name, null, {value:0} )) {
		    replaceOk = true;
		    for( i = 0; i < this.stylesArray.length; i++ ) {
			// check if name already exists
			if( name.value == this.stylesArray[i].name ) {
			    styleExists = bmhLib.getLocaleString( "StyleExistsString" );
			    styleAlready = bmhLib.getLocaleString( "StyleAlreadyString" );
			    replaceOk = promptService.confirm( window, styleExists, 
				    name.value + " " + styleAlready );
			    if( replaceOk ) {
				// change style belonging to name
				this.changeSheetProperty( name.value, i, sheet );
				return true;
			    }
			    break;
			}
		    }
		    if( replaceOk ) {
			// if user has not cancelled replacement, name does not exist yet 
			// and should be added
			this.saveStyleAs( name.value, sheet );
			return true;
		    }
		}
		return false;
	    },

saveAsPrompt: function() {
		  try {
		      this.savePrompt( this.getCurStyleName(), this.stylesheetField.value );
		  }
		  catch( e ) {}
	      },

newPrompt: function() {
	       if( this.savePrompt( "", "" )) {
		   this.stylesheetField.value = "";
		   this.updateStyle( 3 );
	       }
	   },


saveStyleAs: function( name, sheetStr ) {
		 // create resource for new style
		 var newStyle = bmhRDF.RDF.GetResource( bmhRDF.stylesNS + name );
		 var nameNode = bmhRDF.RDF.GetLiteral( name );
		 var sheetNode = bmhRDF.RDF.GetLiteral( sheetStr );
		 var stylesList = bmhRDF.RDF.GetResource( bmhRDF.stylesNS + "stylesList" );
		 bmhRDF.RDFC.Init( this.stylesDS, stylesList );
		 // append new style to styles list in styles datasource
		 bmhRDF.RDFC.AppendElement( newStyle );  
		 // set properties in datasource of new style 
		 this.stylesDS.Assert( newStyle, bmhRDF.nameArc, nameNode, true ); 
		 this.stylesDS.Assert( newStyle, bmhRDF.sheetArc, sheetNode, true ); 
		 var stylesMenupopup; 
		 // update styles array
		 this.stylesArray.push( { name: name, sheet: sheetStr } );
		 // update styles list in options window 
		 stylesMenupopup = document.getElementById( "sheetNames" ); 
		 var stylesMenuitem = document.createElement( "menuitem" );
		 stylesMenuitem.setAttribute( "label", name );
		 stylesMenupopup.appendChild( stylesMenuitem );
		 this.stylesMenu.selectedIndex = this.stylesArray.length - 1;
	     },

deletePrompt: function() {
		  //var deletestyle = document.getElementById( "bmhStrings" ).getString( "DeleteStyleString" );
		  var deletestyle = bmhLib.getLocaleString( "DeleteStyleString" );
		  //var deleteconfirm = document.getElementById( "bmhStrings" ).getString( "DeleteConfirmString" );
		  var deleteconfirm = bmhLib.getLocaleString( "DeleteConfirmString" );

		  var name = this.getCurStyleName();
		  var promptService = this.getPromptService();
		  if( promptService.confirm( window, deletestyle, 
			      deleteconfirm + " " + name + "?" )) {
		      this.deleteStyle();
		  }
	      },

deleteStyle: function() {
		 var index = this.stylesMenu.selectedIndex;
		 var name = this.stylesArray[index].name;
		 // name should be unique
		 var styleResource = bmhRDF.RDF.GetResource( bmhRDF.stylesNS + name );
		 var nameNode = this.stylesDS.GetTarget( styleResource, bmhRDF.nameArc, true );
		 var sheetNode = this.stylesDS.GetTarget( styleResource, bmhRDF.sheetArc, true );
		 // delete all properties of style from datasource 
		 this.stylesDS.Unassert( styleResource, bmhRDF.nameArc, nameNode );
		 this.stylesDS.Unassert( styleResource, bmhRDF.sheetArc, sheetNode );
		 // get stylesList
		 var stylesList = bmhRDF.RDF.GetResource( bmhRDF.stylesNS + "stylesList" );
		 bmhRDF.RDFC.Init( this.stylesDS, stylesList );
		 // remove style from list
		 bmhRDF.RDFC.RemoveElement( styleResource, true );  
		 // update styles array
		 this.stylesArray.splice( index, 1 );
		 // update options window
		 var stylesMenupopup = document.getElementById( "sheetNames" );
		 stylesMenupopup.removeChild( stylesMenupopup.childNodes[index] );
		 var nrOfStyles = this.stylesArray.length;
		 this.stylesMenu.selectedIndex = ( nrOfStyles + index - 1 ) % nrOfStyles;
		 this.changeStyle();
	     },

getCurStyleName: function() {
		     var index = this.stylesMenu.selectedIndex;
		     return this.stylesArray[index].name;
		 },

showColor: function( color ) {
	       document.getElementById( "hexColor" ).value = color;
	   },

changeStyle: function() {
		 this.stylesheetField.value = this.stylesArray[this.stylesMenu.selectedIndex].sheet;
		 this.updateStyle( 3 );
	     },

parseBookmarks: function() {
		    var curName = bmhLib.getProperty( bmhRDF.bmRoot, bmhRDF.NameArc, bmhRDF.BMDS );
		    var curType;
		    var curFolder = { node: bmhRDF.bmRoot, ancestors: [curName] };
		    var folderStack = new Array( curFolder );
		    var inOrExcludedFolders = bmhLib.getArrayFromPref( "inOrExcludedFolders" );
		    var excludePref = bmhLib.getPref( "excludeIndex" );
		    var exclude = ( excludePref == 0 ? true : false );
		    var enumerator, tempFolders, curNode; 
		    var curNodeAncestors, isChecked, isExcluded, i;

		    while( folderStack.length > 0 ) {
			curFolder = folderStack.pop();
			bmhRDF.RDFC.Init( bmhRDF.BMDS, curFolder.node );
			enumerator = bmhRDF.RDFC.GetElements();
			tempFolders = new Array();
			while( enumerator.hasMoreElements() ) {
			    curNode = enumerator.getNext();
			    curType = bmhLib.getProperty( curNode, bmhRDF.typeArc, bmhRDF.BMDS )  
				.split( "#" )[1]; 
			    if( curType == "Folder" || curType == "Livemark" ) {
				curName = bmhLib.getProperty( curNode, bmhRDF.NameArc, bmhRDF.BMDS );
				curNodeAncestors = curFolder.ancestors.concat( [curName] );
				tempFolders.push( { node: curNode, ancestors: curNodeAncestors } );
			    }
			}
			isExcluded = bmhLib.isExcludedFolder( curFolder.ancestors.toString(),
				excludePref, inOrExcludedFolders );
			isChecked = ( exclude == isExcluded ? true : false );
			this.createFolder( curFolder.ancestors, isChecked ); 
			// reverse to match order in bookmarks tree
			folderStack = folderStack.concat( tempFolders.reverse() );
		    }
		},

createFolder: function( ancestors, isChecked ) {  
		  // hbox for checkbox and indentation spacer
		  var folderBox = this.contentFrame.document.createElement( "hbox" );
		  this.inOrExcludedFoldersBox.appendChild( folderBox );
		  // spacer
		  var indentFolderSpacer = this.contentFrame.document.createElement( "spacer" );
		  indentFolderSpacer.setAttribute( "style", "width:" + ((ancestors.length-1)*30) + "px" );
		  folderBox.appendChild( indentFolderSpacer );
		  // checkbox
		  var folderCheckbox = this.contentFrame.document.createElement( "checkbox" );
		  folderCheckbox.setAttribute( "id", "fcb" + this.folderCheckboxArray.length );
		  folderCheckbox.setAttribute( "label", ancestors[ancestors.length-1] );
		  folderCheckbox.setAttribute( "checked", isChecked );
		  folderBox.appendChild( folderCheckbox );
		  // add checkbox to array
		  this.folderCheckboxArray.push( { node: folderCheckbox, value: ancestors } );
	      },

checkSubfolders: function( node ) {
		     if( document.getElementById( "subfolderCheckbox" ).checked ) {
			 // id is "fcb" + index
			 var startIndex = parseInt( node.id.substr( 3 ));
			 var regExp = new RegExp( "^" + this.folderCheckboxArray[startIndex].value );
			 var i = startIndex + 1;
			 while( i < this.folderCheckboxArray.length &&
				 regExp.test( this.folderCheckboxArray[i].value )) {
			     this.folderCheckboxArray[i].node.checked = 
				 this.folderCheckboxArray[startIndex].node.checked;
			     i++;
			 }
		     }
		 }

}

